// -*- C++ -*-
/*!
 * @file ManagerConfig.h
 * @brief RTC manager configuration
 * @date $Date: 2007/04/13 18:02:28 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2005
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: ManagerConfig.h,v 1.4 2007/04/13 18:02:28 n-ando Exp $
 *
 */

/*
 * $Log: ManagerConfig.h,v $
 * Revision 1.4  2007/04/13 18:02:28  n-ando
 * Some configuration properties handling processes were changed.
 *
 * Revision 1.3  2006/11/06 01:26:28  n-ando
 * Some trivial fixes.
 *
 * Revision 1.2  2006/10/23 08:38:23  n-ando
 * To get Property after "init()" calling, getConfig() was added.
 *
 * Revision 1.1  2006/10/17 10:21:38  n-ando
 * The first commitment.
 *
 *
 */


#ifndef ManagerConfig_h
#define ManagerConfig_h

#include <rtm/Properties.h>


namespace RTC
{

  
  /*!
   * @if jp
   *
   * @class ManagerConfig
   * @brief Manager configuration NX
   *
   * Manager ̃RtBM[VsAR}hC󂯎A
   * (邢͈Ȃ)CX^XBAManager ̃vpeB̑Oݒ
   * s
   *
   * ݒ(t@C)̎w͈̋ȉ̂ƂłB
   * オƂAƂアB
   * <OL>
   * <LI>R}hCIvV "-f"
   * <LI>ϐ "RTC_MANAGER_CONFIG"
   * <LI>ftHgݒt@C "./rtc.conf"
   * <LI>ftHgݒt@C "/etc/rtc.conf"
   * <LI>ftHgݒt@C "/etc/rtc/rtc.conf"
   * <LI>ftHgݒt@C "/usr/local/etc/rtc.conf"
   * <LI>ftHgݒt@C "/usr/local/etc/rtc/rtc.conf"
   * <LI>ߍ݃RtBM[Vl
   *</OL>
   * AR}hCIvV "-d" w肳ꂽꍇ́A
   * (Ƃ -f Őݒt@Cw肵Ă)ߍ݃RtBM[Vl
   * gpB
   *
   * @else
   *
   * @brief
   *
   * @endif
   */
  class ManagerConfig
  {
  public:
    // The list of default configuration file path.
    static const char* config_file_path[];
    
    // Environment value to specify configuration file
    static const char* config_file_env;


    /*!
     * @if jp
     *
     * @brief ManagerConfig RXgN^
     *
     * ȂRXgN^B
     *
     * @else
     *
     * @brief ManagerConfig constructor
     *
     * Do nothing. 
     *
     * @endif
     */
    ManagerConfig();


    /*!
     * @if jp
     *
     * @brief ManagerConfig RXgN^
     *
     * ^ꂽɂ菉ɂRXgN^B
     *
     * @param argc R}hC̐
     * @param argv R}hC
     *
     * @else
     *
     * @brief ManagerConfig constructor
     *
     * The constructor that performs initialization at the same time with
     * given arguments.
     *
     * @param argc The number of command line arguments
     * @param argv The command line arguments
     *
     * @endif
     */
    ManagerConfig(int argc, char** argv);


    /*!
     * @if jp
     *
     * @brief ManagerConfig fXgN^
     *
     * @else
     *
     * @brief ManagerConfig destructor
     *
     * @endif
     */
    virtual ~ManagerConfig();
    

    /*!
     * @if jp
     *
     * @brief 
     *
     * ftHgT[`pXRtBM[Vt@CT
     * sƂƂɃvpeBԂBT[`pXɃRtBM[V
     * t@C݂ȂꍇAftHg̃RtBM[V
     * ԂB
     *
     * @return ꂽ Property l
     *
     * @else
     *
     * @brief Initialization
     *
     * This operation searches the configuration file from default search path,
     * and initialize and return default properties.
     * If there is no configuration file in the default search path,
     * default configuration statically defined is used.
     *
     * @return Initialized Property value.
     *
     * @endif
     */
    //    void init();



    /*!
     * @if jp
     *
     * @brief 
     *
     * R}hC^ďBR}hCIvV
     * ȉ̂̂gp\łB
     *
     * -f file   : RtBM[Vt@Cw肷B<br>
     * -l module : [h郂W[w肷B<br>
     * -o options: ̑IvVw肷BB<br>
     * -d        : ftHg̃RtBM[VgB<br>
     *
     * @else
     *
     * @brief Initialization
     *
     * Initialize with command line options. The following command options
     * are available.
     *
     * -f file   : Specify a configuration file. <br>
     * -l module : Specify modules to be loaded at the beginning. <br>
     * -o options: Other options. <br>
     * -d        : Use default static configuration. <br>
     *
     * @endif
     */
    void init(int argc, char** argv);


    /*!
     * @if jp
     * @brief Configuration ̌ʂPropertyɔf
     * @else
     * @brief Apply configuration results to Property
     * @endif
     */
    void configure(Properties& prop);


    /*!
     * @if jp
     *
     * @brief RtBM[V擾
     *
     * RtBM[V擾Binit()ĂяoOɌĂԂƁA
     * ÓIɒ`ꂽftHg̃RtBM[VԂB
     * init() ĂяoɌĂԂƁAR}hCAϐ
     * ÂꂽRtBM[VԂB
     *
     * @else
     *
     * @brief Get configuration value.
     *
     * This operation returns default configuration statically defined,
     * when before calling init() function. When after calling init() function,
     * this operation returns initialized configuration value according to
     * command option, environment value and so on.
     *
     * @endif
     */
    Properties getConfig() const;

  protected:
    /*!
     * @if jp
     *
     * @brief R}hp[X
     *
     * -f file   : RtBM[Vt@Cw肷B<br>
     * -l module : [h郂W[w肷BB<br>
     * -o options: ̑IvVw肷BB<br>
     * -d        : ftHg̃RtBM[VgB<br>
     *
     * @else
     *
     * @brief Parse command arguments
     *
     * -f file   : Specify a configuration file. <br>
     * -l module : Specify modules to be loaded at the beginning. <br>
     * -o options: Other options. <br>
     * -d        : Use default static configuration. <br>
     *
     * @endif
     */
    void parseArgs(int argc, char** argv);
    
    
    /*!
     * @if jp
     *
     * @brief Configuration file T
     *
     * Configuration file ̗D揇
     *
     * R}hIvVw聄ϐftHgt@CftHgݒ
     *
     * ftHgIvV(-d): ftHgt@CĂ
     *                               ftHgݒg
     *
     *
     * @else
     *
     * @brief Find configuration file
     *
     * @endif
     */
    bool findConfigFile();

    
    /*!
     * @if jp
     *
     * @brief VXeZbg
     *
     * VXe擾vpeBɃZbgBݒ肳L[͈ȉ̒ʂB
     * manager.os.name    : OS
     * manager.os.release : OS[X
     * maanger.os.version : OSo[W
     * manager.os.arch    : OSA[LeN`
     * manager.os.hostname: zXg
     * manager.pid        : vZXID
     * 
     * @else
     * 
     * @brief Set system information
     * 
     * Get the following system info and set them to Manager's properties.
     * manager.os.name    : OS name
     * manager.os.release : OS release name
     * maanger.os.version : OS version
     * manager.os.arch    : OS architecture
     * manager.os.hostname: Hostname
     * manager.pid        : process ID
     *
     * @endif
     */
    void setSystemInformation(Properties& prop);


    /*!
     * @if jp
     * @brief t@C݂邩ǂm߂
     * @else
     * @brief Check file existance
     * @endif
     */
    bool fileExist(const std::string& filename);

    std::string m_configFile;
  };
  
}; // namespace RTC  
  
  
#endif // ManagerConfig_h
  
